package com.ibm.iru.db2express9_1fixpacklnx;

/**
 * @(#)DB2Express9_1FixpackLnxMain.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set forth
 * in the license agreement document for the IBM Express Runtime.
 */

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * 
 * This class updates DB2 base installation with the apropriate fixpack level.
 *
 */
public class DB2Express9_1FixpackLnxMain extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private static final int RC_SUCCESS_WITH_WARNINGS = 1;

  private String ivMediaFileName = null;
  private String ivNLPackFileName = null;
  private SupportLinuxHelper ivHelper;
  private String ivInstallDir = null;

  /**
   * Expect one or two arguments,
   * @param  args[0] - The name of the fixpack image tar or gz file
   *         args[1] - The name of the DB2 National Laungage Pack (nlpack) tar or gz file
   * 
   */
  private int init(String args[])
  {
    int rc = SUCCESS;

    // Set initial data
    setMainResources("com.ibm.iru.message.MessagesNLS");
    ivHelper = getLinuxHelper();
    int no_Of_Expected_args = 1;
    //installation directory
    ivInstallDir = ivHelper.getIbmNsiPropValue("installDir");

    //command to verify whether installed version of DB2 is non-English. Return code zero indicates that DB2 is non-English.
    String cmd = "db2ls -q -b " + ivInstallDir
        + " -a -c | awk '$1~/^DB2_PRODUCT_MESSAGE/{print$1}' FS=: | grep -v _EN$";
    
    if (ivHelper.invokeCommandLogOutput(cmd) == 0)
    {
      //DB2 National Language Pack is required
      no_Of_Expected_args = 2;
    }

    //arguments if provided more than expected will be ignored
    if (args.length ==  no_Of_Expected_args || args.length > no_Of_Expected_args)
    {
      if (no_Of_Expected_args == 1)
      {
        //Set first argument to the fix pack image file name.
        ivMediaFileName = args[0].trim();
      }
      else
      {
        //Set first argument to the fix pack image file name.
        ivMediaFileName = args[0].trim();
        //set second argument to the DB2 National Laungage Pack
        ivNLPackFileName = args[1].trim();
      }
    }
    else
    {
      //an incorrect number of arguments were passed log the default values that were used.
      String[] replacementText = { "DB2Express9_1FixpackLnxMain", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.postErrorMessageToDeployerUI(this);
      rc = FAILURE;
    }
    return rc;
  }

  /**
   * performUpgrade() - Build up the command, and run it.
   *
   * @return - SUCCESS if successful, FAILURE if exception occurred, or the
   *           command did not return successfully.
   */
  private int performUpgrade()
  {
    int rc = SUCCESS;

    // Extract the image tar file
    String unpackedDir = ivHelper.getUnpackedDir(this);
    //for the case where the image file has ".gz" extension
    if (ivMediaFileName.endsWith("gz"))
    {
      String cmd1 = "gunzip " + unpackedDir + ivMediaFileName;
      ivHelper.invokeCommandLogOutput(cmd1);
      ivMediaFileName = ivMediaFileName.substring(0, ivMediaFileName.indexOf(".gz"));
    }
    String command = "tar xf " + unpackedDir + ivMediaFileName + " -C " + unpackedDir;
    rc = ivHelper.invokeCommandLogOutput(command);

    //extract the DB2 National Laungage Pack
    if (SUCCESS == rc)
    {
      if (ivNLPackFileName != null && !ivNLPackFileName.equals(""))
      {
        //for the case where the language package name ends with ".gz"
        if (ivNLPackFileName.endsWith("gz"))
        {
          String cmd1 = "gunzip " + unpackedDir + ivNLPackFileName;
          ivHelper.invokeCommandLogOutput(cmd1);
          ivNLPackFileName = ivNLPackFileName.substring(0, ivNLPackFileName.indexOf(".gz"));
        }
        //untar the language pack
        String cmd2 = "tar -xvf " + unpackedDir + ivNLPackFileName;
        rc = ivHelper.invokeCommandLogOutput(cmd2);
      }
    }

    if (SUCCESS == rc)
    {
      setFileName(SLASH + "tmp" + SLASH + "DB2FixpackInstall.log"); //sets the log file name
      String cmd = unpackedDir + "exp" + SLASH + "disk1" + SLASH + "installFixPack -b " + ivInstallDir + " -l "
          + getFileName();
      rc = ivHelper.invokeCommandLogOutput(cmd);

      // check if warnings occurred during install
      if (RC_SUCCESS_WITH_WARNINGS == rc)
      {
        // indicate that a warning condition was encountered.
        setMessage(getResourceString(NLSKeys.INSTALL_WARNING));
        ivHelper.postWarningMessageToDeployerUI(this);

        // since warnings are not critical, reset rc to success
        // to allow install to continue
        rc = SUCCESS;
      }

      //Append the install log file
      if (ivHelper.fileExists(this))
      {
        // Append the install log
        ivHelper.logAppendFile(this);
      }
      else
      {
        //the log file was not found
        setMessage(getResourceString("LOG_FILE_NOT_FOUND", getFileName()));
        ivHelper.postErrorMessageToDeployerUI(this);
        rc = FAILURE;
      }
    }
    return rc;
  }

  /**
   * Perform post upgrade steps.
   * <li>1. Update instances to use the new level of DB2
   * <li>2. Update DB2 Administration Server (DAS) to use the new level of DB2
   * <li>3. Restart the instances
   * @return SUCCESS if the post Upgrade steps completed successfully, FAILURE otherwise
   */
  private int doPostUpgradeSteps()
  {
    DB2InstalledProduct db2Product = DB2InstalledProduct.getInstalledProduct(this, ivInstallDir);
    int rc = db2Product.updateInstances(ivInstallDir);
    if (rc != FAILURE)
    {
      rc = db2Product.updateDAS(ivInstallDir);
    }
    if (rc != FAILURE)
    {
      rc = db2Product.startDB2(ivInstallDir);
    }
    if (rc != FAILURE)
    {
      rc = db2Product.startDAS();
    }
    return rc;
  }

  /**
   * DB2 Express Fixpack main program.
   * @param  args[0] - The name of the tar file 
   *         args[1] - The name of the national language package (nlpack) tar file
   *
   */
  public static void main(String[] args)
  {
    // Create and initialize the object
    DB2Express9_1FixpackLnxMain main = new DB2Express9_1FixpackLnxMain();
    int rc = main.init(args);

    // Install DB2 fixpack
    if (rc == SUCCESS)
    {
      rc = main.performUpgrade();
    }
    //If successful, do post-install steps required by DB2 Linux
    if (rc == SUCCESS)
    {
      rc = main.doPostUpgradeSteps();
    }

    System.exit(rc);
  }
}