package com.ibm.iru.db2express9_1fixpackwin;

/**
 * @(#)DB2Express9_1FixpackWinMain.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the IBM Express Runtime.
 */

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportWindowsBase;
import com.ibm.jsdt.support.SupportWindowsHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;


/**
 * This class launches the DB2 Express Fix pack.
 *
 */
public class DB2Express9_1FixpackWinMain extends SupportWindowsBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private static final int RC_SUCCESS_WITH_WARNINGS = 1;
  private static final int RC_SUCCESS_NEEDS_REBOOT = 3010;
  private SupportWindowsHelper ivHelper;
  private String ivInstallDir = null;
    
  /**
   * init method has input parameters:
   * parm#0 = response file name
   */
  private int init(String[] args)
  {
    int rc = SUCCESS;
    setMainResources("com.ibm.iru.message.MessagesNLS");
    ivHelper = getWindowsHelper();
    
    // Ensure the right number of parameters was passed
    int no_Of_Expected_args = 1;
    if (no_Of_Expected_args == args.length)
    {
      setResponseFileName(args[0]);
      //installation directory
      setKey("FILE");
      ivInstallDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    }
    else
    {
      String[] replacementText = { "DB2Express9_1FixpackWinMain", String.valueOf(no_Of_Expected_args), String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.log(this);
      rc = FAILURE;
    }
    
    return rc;
  }

  /**
   * Invoke the install of the fixpack. Verify the install, by
   * checking existence in the registry
   * @return SUCCESS -if the install was successful, FAILURE otherwise
   */
  private int performUpgrade()
  {
    int rc = SUCCESS;
    //if installDir already has a "\" appended to the end, don't reappend it
    if (!ivInstallDir.endsWith(SLASH))
    {
      ivInstallDir = ivInstallDir + SLASH;
    }
    
    setFileName(ivInstallDir + "DB2FixpackInstall.log");
    String command = "cmd /c " + ivHelper.getUnpackedDir(this) + "setup.exe /F /U " + getResponseFileName()
        + " /L " + "\"" + getFileName() + "\"";
    rc = ivHelper.invokeCommandLogOutput(command);
     
    // Append the install log file
    if (ivHelper.fileExists(this))
    {
      // Append the install log
      ivHelper.logAppendFile(this);
    }
    else
    {
      setMessage(getResourceString("LOG_FILE_NOT_FOUND", getFileName()));
      ivHelper.postErrorMessageToDeployerUI(this);
      
      setMessage(getResourceString(NLSKeys.CMD_OUTPUT, ivHelper.getLastCommandOutput(this)));
      ivHelper.log(this);
      
      rc = FAILURE;
    }

    // check if warnings occurred during install OR if a reboot is necessary.
    if (RC_SUCCESS_WITH_WARNINGS == rc)
    {
      // indicate that a warning condition was encountered.
      setMessage(getResourceString(NLSKeys.INSTALL_WARNING));
      ivHelper.postWarningMessageToDeployerUI(this);
      // since warnings are not critical, reset rc to success to allow
      // install to continue
      rc = SUCCESS;
    }
    else if (rc == RC_SUCCESS_NEEDS_REBOOT)
    {
      //indicate that a reboot is needed.
      setMessage(getResourceString(NLSKeys.REBOOT_REQUIRED));
      ivHelper.postWarningMessageToDeployerUI(this); 
      rc = SUCCESS;
    }
    return rc;
  }

  /**
   * Perform post upgrade steps.
   * Restart the instances
   * @return SUCCESS if the post Upgrade steps completed successfully, FAILURE otherwise
   */
  private int doPostUpgradeSteps()
  {
    //remove extra slash added earlier
    if (ivInstallDir.endsWith(SLASH))
    {
      int endIndex = ivInstallDir.lastIndexOf(SLASH);
      ivInstallDir = ivInstallDir.substring(0, endIndex);
    }
    DB2InstalledProduct db2Product = DB2InstalledProduct.getInstalledProduct(this, ivInstallDir);
    int rc = db2Product.startDB2(ivInstallDir);
    if (rc != FAILURE)
    {
      rc = SUCCESS;
    }
    return rc;
  }


  /**
   * The entry point for the DB2 Express Fixpack main program.
   * @param args - response file name
   */
  public static void main(String[] args)
  {
    // Create and initialize the object
    DB2Express9_1FixpackWinMain main = new DB2Express9_1FixpackWinMain();
    int rc = main.init(args);

    // Do the upgrade
    if (rc == SUCCESS)
    {
      rc = main.performUpgrade();
    }
    //If successful, do post-install steps
    if (rc == SUCCESS)
    {
      rc = main.doPostUpgradeSteps();
    }

    System.exit(rc);
  }
}
