/**
 * WAS6_1FixpackI5OSMain.java
 * 
 * Licensed Materials - Property of IBM
 * 
 * 5724-J10
 * 
 * (C) Copyright IBM Corp. 2007
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * This is sample code made available for use in accordance with terms set 
 * forth in the license agreement document for the IBM Express Runtime.
 */
package com.ibm.iru.was6_1fixpacki5os;

import java.io.File;
import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportOS400Base;
import com.ibm.jsdt.support.SupportOS400Helper;
import com.ibm.jsdt.support.installedproduct.WASExpressInstalledProduct;

/**
 * This class performs the actual install.
 */
public class WAS6_1FixpackI5OSMain extends SupportOS400Base
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private SupportOS400Helper ivHelper;
  private String ivInstallDir = null;
  private String ivLogFile = null;
  private WASExpressInstalledProduct ivWASProduct = null;
  private String ivMaintanencePak = null;
  //success string in the log file
  String SUCCESS_SEARCH_STRING = "INSTCONFSUCCESS";
  String WARNING_SEARCH_STRING = "INSTCONFPARTIALSUCCESS";
  //following are the prefix WebSphere Application Server - Express fix pack name usually available with. 
  //Change this value if fix pack file has specified with a different prefix
  String WAS_FIXPACK_FILE_PREFIX = "6.1.0-WS-WAS-i5osPPC";

  /**
   * Initializes the framework and validates arguments
   * Expect one argument - update installer response file
   * 
   * @return SUCCESS if initialization successful
   *         FAILURE if argument incorrect or other initialization failed
   */
  private int init(String[] args)
  {
    int no_Of_Expected_Args = 1;
    int returnCode = SUCCESS;

    ivHelper = getOS400Helper();
    // set the message bundle and jar file for the resource strings
    setMainResources("com.ibm.iru.message.MessagesNLS");

    //Ensure the right number of parameters was passed
    if (args.length != no_Of_Expected_Args)
    {
      String[] replacementText = { "WASExpress6_1FixpackI5OSMain", String.valueOf(no_Of_Expected_Args),
          String.valueOf(args.length) };
      ivHelper.log(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      returnCode = FAILURE;
    }
    else
    {
      setResponseFileName(args[0]);
      ivInstallDir = ivHelper.getIbmNsiPropValue("installationLocationWas");
      if (ivInstallDir.endsWith("/"))
      {
        ivInstallDir = ivInstallDir.substring(0, ivInstallDir.lastIndexOf("/"));
      }

      ivWASProduct = WASExpressInstalledProduct.getInstalledProduct(this, ivInstallDir);
      //get .pak files list from the ER unpacked directory
      String[] pakFilesArray = ivHelper.directoryGrep(ivHelper.getUnpackedDir(this), ".pak");
      if (pakFilesArray == null || pakFilesArray.length < 1)
      {
        setMessage("Could not find fix pack file with extension .pak.");
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
      else if (pakFilesArray.length == 1)
      {
        ivMaintanencePak = pakFilesArray[0];
      }
      else
      {
        boolean pakFound = false;
        for (int i = 0; i < pakFilesArray.length && !pakFound; i++)
        {
          if (pakFilesArray[i].indexOf(WAS_FIXPACK_FILE_PREFIX) > -1)
          {
            ivMaintanencePak = pakFilesArray[i];
            pakFound = true;
          }
        }
        //fix pack file not found with the expected prefix. 
        if (!pakFound)
        {
          setMessage("Could not find fix pack file with expected name. Expected file name prefix is "
              + WAS_FIXPACK_FILE_PREFIX + ".");
          ivHelper.postErrorMessageToDeployerUI(this);
          returnCode = FAILURE;
        }
      }

      String fixpakInstallLogDirName = ivMaintanencePak.substring(0, ivMaintanencePak.indexOf(".pak")) + ".install";
      ivLogFile = ivInstallDir + SLASH + "logs" + SLASH + "update" + SLASH + fixpakInstallLogDirName + SLASH
          + "updatelog.txt";
    }
    return returnCode;
  }

  /**
   * Run the following procedures:
   * 1. Install UpdateInstaller.
   * 2. Copy the fixpack into the maintenance directory of UpdateInstaller.
   * 3. Install the fixpack.
   * 4. Run the install verification tool on the instance name provided.
   * 
   * @return SUCCESS if update installer and fix pack installation successful 
   *         FAILURE otherwise
   */
  private int run()
  {
    int returnCode = SUCCESS;
    String unpackedDir = ivHelper.getUnpackedDir(this);
    setKey("-OPT installLocation");
    String updiInstallLocation = ivHelper.getResponseFileValue(this);
    //remove quoutes from the installation directory
    updiInstallLocation = updiInstallLocation.replaceAll("\"", "").trim();

    //if the install log exist, get the size to be used later for checking.  Note this is not the UpdateInstaller log file.
    File logFile = new File(ivLogFile);
    long logStartSize = 0;
    if (logFile.exists())
    {
      logStartSize = logFile.length();
    }

    if (returnCode != FAILURE)
    {
      //Install Update installer
      String commandToInstall = "qsh cmd('" + unpackedDir + "UpdateInstaller" + SLASH + "install -options "
          + unpackedDir + getResponseFileName() + "')";
      if (invokeCLCommand(commandToInstall, commandToInstall) == FAILURE
          || checkLogs(0, updiInstallLocation + SLASH + "logs" + SLASH + "install" + SLASH + "log.txt") == FAILURE)
      {
        setMessage(getResourceString(NLSKeys.WAS_FIXPACK_FAIL_TO_INSTALL_UPDATE_INSTALLER));
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
      if (returnCode != FAILURE)
      {
        //Copy the fixpack to the location of Update Installer
        String copyCommand = "qsh cmd('cp " + unpackedDir + ivMaintanencePak + " " + updiInstallLocation + SLASH
            + "maintenance')";
        if (invokeCLCommand(copyCommand, copyCommand) == FAILURE)
        {
          setMessage(getResourceString(NLSKeys.WAS_FIXPACK_FAIL_TO_COPY_FIXPACK));
          ivHelper.postErrorMessageToDeployerUI(this);
          returnCode = FAILURE;
        }
        if (returnCode != FAILURE)
        {
          //Install the fixpack.
          String fixpackInstallCommand = "qsh cmd('" + updiInstallLocation + SLASH + "update -W product.location="
              + ivInstallDir + " -W maintenance.package=" + updiInstallLocation + SLASH + "maintenance" + SLASH
              + ivMaintanencePak + "')";
          if (invokeCLCommand(fixpackInstallCommand, fixpackInstallCommand) == FAILURE
              || checkLogs(logStartSize, ivLogFile) == FAILURE)
          {
            setMessage(getResourceString(NLSKeys.WAS_FIXPACK_FAIL_TO_INSTALL));
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
          }
          if (returnCode != FAILURE)
          {
            //Run install verification test
            int rc = ivWASProduct.runInstallVerificationTestUsingDefualtProfileAndServer();
            if (rc == FAILURE)
            {
              setMessage(getResourceString(NLSKeys.WAS_VERIFICATION_FAIL));
              ivHelper.postErrorMessageToDeployerUI(this);
              returnCode = FAILURE;
            }
          }
        }
      }
    }
    return returnCode;
  }

  /**
   * Look for the string INSTCONFSUCCESS: in WAS install log file
   * for success indicator.  Look for the string INSTCONFPARTIALSUCCESS
   * in the WAS install log and log as a for warning if found.
   * Also, append the install log to the main log.
   * 
   * @param startLogSize - The place in the log file to start looking for the success string.
   * @param logFile - The name of the log file.
   * @return SUCCESS - indicates a successful install
   *         FAILURE - indecates the install failed.
   */
  private int checkLogs(long startLogSize, String logFile)
  {
    int returnCode = FAILURE;
    if (new File(logFile).exists())
    {
      ivHelper.logAppendFile(logFile);
      //search for success message
      setGrepFileName(logFile);
      setGrepString(SUCCESS_SEARCH_STRING);
      setFileTextStartIndex((new Long(startLogSize)).intValue());
      if (ivHelper.fileGrep(this))
      {
        returnCode = SUCCESS;
      }
      //search for any warning messages
      setGrepString(WARNING_SEARCH_STRING);
      if (ivHelper.fileGrep(this))
      {
        setMessage(getResourceString(NLSKeys.WAS_FIXPACK_WARNING_IN_LOG_FILE, logFile));
        ivHelper.postWarningMessageToDeployerUI(this);
        returnCode = SUCCESS;
      }
    }
    else
    {
      setMessage(getResourceString("LOG_FILE_NOT_FOUND", logFile));
      ivHelper.postErrorMessageToDeployerUI(this);
    }

    return returnCode;

  }

  /**
   * Run a CL command.
   * 
   * @param command - the command to run, logging parameter.
   * @param logCommand - the command to display in the log file.
   * 
   * @return SUCCESS, FAILURE
   */
  public int invokeCLCommand(String command, String logCommand)
  {
    int returnCode = SUCCESS;
    ivHelper.logNewLine(this);
    setMessage(getResourceString("COMMAND_INVOKED", logCommand));
    ivHelper.log(this);

    //set and call the CL command using the framework
    setCommand(command);

    if (ivHelper.runCLCommand(this))
    {
      setMessage(getResourceString(NLSKeys.CMD_SUCCESS));
      ivHelper.log(this);
    }
    else
    {
      setMessage(getResourceString(NLSKeys.CMD_FAIL, new String[] { String.valueOf(ivHelper
          .getLastCommandReturnCode(this)) }));
      ivHelper.log(this);
      returnCode = FAILURE;
    }
    setMessage(ivHelper.getLastCommandOutput(this));
    ivHelper.log(this);
    return returnCode;
  }

  /**
   * Main entry point.
   * 
   * @param args - expects one argument 
   *               1. response file for the update installer
   * 
   *  
   */
  public static void main(String[] args)
  {
    //create and initialize the object
    WAS6_1FixpackI5OSMain main = new WAS6_1FixpackI5OSMain();

    //initialize class variables
    int returnCode = main.init(args);
    if (returnCode == SUCCESS)
    {
      returnCode = main.run();
    }
    System.exit(returnCode);
  }
}