/**
 * WAS6_1FixpackI5OSPDC.java
 * 
 * Licensed Materials - Property of IBM
 * 
 * 5724-J10
 * 
 * (C) Copyright IBM Corp. 2007
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * This is sample code made available for use in accordance with terms set forth
 * in the license agreement document for the IBM Express Runtime.
 */
package com.ibm.iru.was6_1fixpacki5os;

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportOS400Base;
import com.ibm.jsdt.support.SupportOS400Helper;
import com.ibm.jsdt.support.installedproduct.WASExpressInstalledProduct;

public class WAS6_1FixpackI5OSPDC extends SupportOS400Base
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private SupportOS400Helper ivHelper;
  private WASExpressInstalledProduct ivWASProduct;

  /**
   * Initializes the framework and validates arguments
   * Expect no argument.
   * 
   * @return SUCCESS if initialization successful
   *         FAILURE if argument incorrect or other initialization failed
   */
  private int init(String[] args)
  {
    int no_Of_Expected_Args = 0;
    int returnCode = SUCCESS;
    ivHelper = getOS400Helper();
    //set the message bundle for the resource strings
    setMainResources("com.ibm.iru.message.MessagesNLS");

    //ensure the right number of parameters was passed
    if (args.length != no_Of_Expected_Args)
    {
      String[] replacementText = { "WASExpress6_1FixpackI5OSPDC", String.valueOf(no_Of_Expected_Args),
          String.valueOf(args.length) };
      ivHelper.log(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      returnCode = FAILURE;
    }
    return returnCode;
  }

  /**
   * Perform the required checks before deploying WebSphere. This includes determining the install action 
   * and verifying/validating the variables.
   * 
   * @return PDC_EXISTS, PDC_DOES_NOT_EXIST or FAILURE
   */
  private int performPredeploymentChecks()
  {
    int returnCode = PDC_DOES_NOT_EXIST;
    String installLocation = ivHelper.getIbmNsiPropValue("installationLocationWas");
    if ((installLocation == null) || installLocation.equals(""))
    {
      //return FAILURE as installation location is a required parameter
      setMessage(getResourceString(NLSKeys.INSTALLATION_LOCATION_NOT_FOUND));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    else
    {
      ivWASProduct = WASExpressInstalledProduct.getInstalledProduct(this, installLocation);
      if (ivWASProduct != null)
      {
        returnCode = compareVersion(ivWASProduct.getInstalledVersion());
        if (returnCode == FAILURE)
        {
          //earlier or later version of WebSphere is installed where fix pack can not be installed.
          setMessage("Fix pack version" + ivHelper.getProductInstallingVersion(this)
              + " cannot be installed on the WebSphere Application Server - Express version "
              + ivWASProduct.getInstalledVersion() + ".");
          ivHelper.postErrorMessageToDeployerUI(this);
          returnCode = FAILURE;
        }
      }
      else
      {
        setMessage("A version of WebSphere Application Server - Express cannot be detected at " + installLocation + ".");
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
    }

    if (returnCode == PDC_DOES_NOT_EXIST)
    {
      //stop servers
      returnCode = stopWASServers();
    }
    return returnCode;
  }

  /**
   * Stop WebSphere default server server1 for all the profiles. If profile found is not default then attemp will made to stop
   * server using the same server name as profile.
   * 
   * @return FAILURE if any of the servers failed to stop.
   *         SUCCESS if all application servers were already stopped or stopped successfully.
   */
  private int stopWASServers()
  {
    int returnCode = SUCCESS;
    boolean stopServerFailure = false;

    String wasAdminUserName = ivHelper.getIbmNsiPropValue("appServerAdminUserName");
    String wasAdminUserPassword = ivHelper.getIbmNsiPropValue("appServerAdminPassword");
    //if a userid is specified, then make sure a password is also there
    if (wasAdminUserName != null && !wasAdminUserName.equals("")
        && (wasAdminUserPassword == null || wasAdminUserPassword.equals("")))
    {
      setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, new String[] { wasAdminUserName }));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    //if password is specified, make sure user id is specified
    if (wasAdminUserPassword != null && !wasAdminUserPassword.equals("")
        && (wasAdminUserName == null || wasAdminUserName.equals("")))
    {
      setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, new String[] { "\"\"" }));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }

    if (returnCode == SUCCESS)
    {
      String[] profiles = ivWASProduct.getProfiles();
      int i = 0;
      while (i < profiles.length)
      {
        String profileName = profiles[i].trim();
        String serverName = profiles[i].trim();
        if (profileName.equalsIgnoreCase("default"))
        {
          serverName = "server1";
        }
        if (ivWASProduct.isWASServerRunning(serverName, profileName, wasAdminUserName, wasAdminUserPassword))
        {
          returnCode = ivWASProduct.stopServer(serverName, profileName, wasAdminUserName, wasAdminUserPassword);
          if (returnCode == FAILURE)
          {
            setMessage(getResourceString(NLSKeys.WAS_STOP_SERVER_INSTANCE_FAILED, serverName));
            ivHelper.postErrorMessageToDeployerUI(this);
            stopServerFailure = true;
          }
        }
        i++;
      }
    }
    if (stopServerFailure)
    {
      returnCode = FAILURE;
    }
    return returnCode;
  }

  /**
   * Compare installed version of middleware with the version being installed.
   * 
   * @param String -
   *            installed version of the middleware
   * 
   * @return PDC_DOES_NOT_EXIST - if fix pack is installable at the specified location.
   *         PDC_EXISTS - if fix pack is already installed at the specified location.
   *         FAILURE - if fix pack is not installable at the specified location.
   */
  private int compareVersion(String installedVersion)
  {
    int returnCode = SUCCESS;
    // trim the version specified by user to first three digit. 
    //This version is conidered as a major version of the middleware for which fix pack installation is needed.
    String wasMajorVersion = ivWASProduct.trimVersionToNVersionPlaces(3, ivHelper.getProductInstallingVersion(this));
    // trim the installed version to first three digit
    String trimmedInstalledVersion = ivWASProduct.trimVersionToNVersionPlaces(3, installedVersion);
    // compare installed version with the version to be installed
    setVersion(installedVersion);
    setCompareVersion(ivHelper.getProductInstallingVersion(this));
    switch (ivHelper.compareVersions(this))
    {
      //version is lower than the version to be installed
      case -1:
      {
        //check if version is 6.1.0.x
        if (trimmedInstalledVersion.equalsIgnoreCase(wasMajorVersion))
        {
          //continue installation
          returnCode = PDC_DOES_NOT_EXIST;
        }
        else
        {
          //a lower version then 6.1.0.0 is installed
          returnCode = FAILURE;
        }
        break;
      }
        //versions are equal
      case 0:
      {
        returnCode = PDC_EXISTS;
        break;
      }
      case 1:
      {
        //later version already installed - no need to continue installation
        returnCode = FAILURE;
        break;
      }
    }
    return returnCode;
  }

  /**
   * Main.  Checks the current state of the target machine for the requirements and for any existing WAS Express installations.
   * 
   * @param args - expect no argument
   */
  public static void main(String args[])
  {
    //create and initialize the object
    WAS6_1FixpackI5OSPDC pdc = new WAS6_1FixpackI5OSPDC();

    //call init and terminate if it fails
    int returnCode = pdc.init(args);
    if (returnCode == SUCCESS)
    {
      returnCode = pdc.performPredeploymentChecks();
    }
    System.exit(returnCode);
  }
}