/**
 * WebSphereFamily6_1FixpackLnxPDC.java
 * 
 * Licensed Materials - Property of IBM
 * 
 * 5724-J10
 * 
 * (C) Copyright IBM Corp. 2007
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * This is sample code made available for use in accordance with terms set forth
 * in the license agreement document for the IBM Express Runtime.
 */
package com.ibm.iru.webspherefamily6_1fixpacklnx;

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import com.ibm.jsdt.support.installedproduct.InstalledProduct;
import com.ibm.jsdt.support.installedproduct.WASExpressInstalledProduct;
import com.ibm.jsdt.support.installedproduct.IHSInstalledProduct;
import com.ibm.jsdt.support.installedproduct.WebserverPluginForWASInstalledProduct;

public class WebSphereFamily6_1FixpackLnxPDC extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private SupportLinuxHelper ivHelper;
  private InstalledProduct ivMiddlewareProduct = null;
  private String ivMiddlewareLocation = null;
  private String ivMiddlewareName = null;
  private String ivIHSLocation = null;
  boolean ivIsWASFixpack = false;
  boolean ivIsIHSFixpack = false;
  boolean ivIsPLGFixpack = false;

  /**
   * Initializes the framework and validates arguments. 
   * Expects no argument.
   * 
   * @return SUCCESS if itialization successful FAILURE if arguments incorrect
   *         or any initialization failed
   */
  private int init(String[] args)
  {
    int no_Of_Expected_Args = 0;
    int returnCode = SUCCESS;
    ivHelper = getLinuxHelper();
    // set the message bundle for the resource strings
    setMainResources("com.ibm.iru.message.MessagesNLS");

    //ensure the right number of parameters was passed
    if (args.length != no_Of_Expected_Args)
    {
      String[] replacementText = { "WebSphereFamily6_1FixpackWinPDC", String.valueOf(no_Of_Expected_Args),
          String.valueOf(args.length) };
      ivHelper.log(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      returnCode = FAILURE;
    }
    else
    {
      ivMiddlewareName = ivHelper.getIbmNsiPropValue("middlewareSelected");
      if (ivMiddlewareName.equalsIgnoreCase("WAS"))
      {
        ivIsWASFixpack = true;
        ivMiddlewareLocation = ivHelper.getIbmNsiPropValue("installationLocationWas");
      }
      else if (ivMiddlewareName.equalsIgnoreCase("IHS"))
      {
        ivIsIHSFixpack = true;
        ivMiddlewareLocation = ivHelper.getIbmNsiPropValue("installationLocationIhs");
      }
      else if (ivMiddlewareName.equalsIgnoreCase("PLG"))
      {
        ivIsPLGFixpack = true;
        ivMiddlewareLocation = ivHelper.getIbmNsiPropValue("installationLocationPlg");
        ivIHSLocation = ivHelper.getIbmNsiPropValue("installationLocationIhs");
        if ((ivIHSLocation == null) || ivIHSLocation.equals(""))
        {
          setMessage("Installation location of IBM HTTP Server is required to install fix pack for the WebSphere plugin.");
          ivHelper.postErrorMessageToDeployerUI(this);
          returnCode = FAILURE;
        }
      }
    }
    return returnCode;
  }

  /**
   * Determine installation action.
   * 
   * @return PDC_DOES_NOT_EXIST if a version detected which can be upgraded
   *         with the fix pack PDC_EXIST if installed version is same as the
   *         one being installed FAILURE if fix pack installation can not be
   *         applied
   */
  private int determineInstallAction()
  {
    int returnCode = SUCCESS;
    String messageFixpackNotInstallable = null;
    String messageMiddlewareNotDetected = null;
    String findInstalledVersion = "installedVersion";

    //if provided middleware location is not empty or null, get an instance of installed middleware
    if ((ivMiddlewareLocation == null) || ivMiddlewareLocation.equals(""))
    {
      // return FAILURE as installation location is a required parameter
      setMessage(getResourceString(NLSKeys.INSTALLATION_LOCATION_NOT_FOUND));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    else
    {
      //get installed product instance and set messages according to middleware selected
      if (ivIsWASFixpack)
      {
        ivMiddlewareProduct = WASExpressInstalledProduct.getInstalledProduct(this, ivMiddlewareLocation);
        messageMiddlewareNotDetected = "A version of WebSphere Application Server - Express cannot be detected at "
            + ivMiddlewareLocation + ".";
        messageFixpackNotInstallable = "Fix pack version " + ivHelper.getProductInstallingVersion(this)
            + " cannot be installed on the WebSphere Application Server - Express version " + findInstalledVersion + ".";
      }
      else if (ivIsIHSFixpack)
      {
        ivMiddlewareProduct = IHSInstalledProduct.getInstalledProduct(this, ivMiddlewareLocation);
        messageMiddlewareNotDetected = "A version of IBM HTTP Server - Express cannot be detected at "
            + ivMiddlewareLocation + ".";
        messageFixpackNotInstallable = "Fix pack version " + ivHelper.getProductInstallingVersion(this)
            + " cannot be installed on the IBM HTTP Server version " + findInstalledVersion + ".";
      }
      else if (ivIsPLGFixpack)
      {
        ivMiddlewareProduct = WebserverPluginForWASInstalledProduct.getInstalledProduct(this, ivMiddlewareLocation);
        messageMiddlewareNotDetected = "A version of WebSphere plugin cannot be detected at " + ivMiddlewareLocation + ".";
        messageFixpackNotInstallable = "Fix pack version " + ivHelper.getProductInstallingVersion(this)
            + " cannot be installed on the WebSphere plugin version " + findInstalledVersion + ".";
      }
    }

    //if an instance of installed middleware is not null, get the installed version and compare it with the version being installed
    if (ivMiddlewareProduct != null)
    {
      returnCode = compareVersion(ivMiddlewareProduct.getInstalledVersion());
      if (returnCode == FAILURE)
      {
        // earlier or later version of WebSphere is installed, fix pack can not be installed.
        setMessage(messageFixpackNotInstallable
            .replace(findInstalledVersion, ivMiddlewareProduct.getInstalledVersion()));
        ivHelper.postErrorMessageToDeployerUI(this);
      }
    }
    else
    {
      setMessage(messageMiddlewareNotDetected);
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    //if fix pack is installable, stop the required servers if running
    if (returnCode == PDC_DOES_NOT_EXIST)
    {
      //stop servers
      if (ivIsWASFixpack)
      {
        returnCode = stopWASServers();
      }
      if (ivIsIHSFixpack || ivIsPLGFixpack)
      {
        returnCode = stopIHSServers();
      }
    }
    return returnCode;
  }

  /**
   * Stop WebSphere default server for all the profiles.
   * 
   * @return FAILURE if any of the application servers failed to stop.
   *         SUCCESS if all application servers were already stopped or stopped successfully.
   */
  private int stopWASServers()
  {
    int returnCode = SUCCESS;
    boolean stopServerFailure = false;

    String wasAdminUserName = ivHelper.getIbmNsiPropValue("appServerAdminUserName");
    String wasAdminUserPassword = ivHelper.getIbmNsiPropValue("appServerAdminPassword");
    //if a userid is specified, then make sure a password is also there
    if (wasAdminUserName != null && !wasAdminUserName.equals("")
        && (wasAdminUserPassword == null || wasAdminUserPassword.equals("")))
    {
      setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, new String[] { wasAdminUserName }));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    //if password is specified, make sure user id is specified
    if (wasAdminUserPassword != null && !wasAdminUserPassword.equals("")
        && (wasAdminUserName == null || wasAdminUserName.equals("")))
    {
      setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, new String[] { "\"\"" }));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }

    if (returnCode == SUCCESS)
    {
      WASExpressInstalledProduct wasProduct = (WASExpressInstalledProduct)ivMiddlewareProduct;
      String[] profiles = wasProduct.getProfiles();
      int i = 0;
      while (i < profiles.length)
      {
        if (wasProduct.isWASServerRunning("server1", profiles[i], wasAdminUserName, wasAdminUserPassword))
        {
          returnCode = wasProduct.stopServer("server1", profiles[i], wasAdminUserName, wasAdminUserPassword);
          if (returnCode == FAILURE)
          {
            setMessage("Failed to stop default WebSphere Application Server - Express for profile " + profiles[i] + ".");
            ivHelper.postErrorMessageToDeployerUI(this);
            stopServerFailure = true;
          }
        }
        i++;
      }
    }
    if (stopServerFailure)
    {
      returnCode = FAILURE;
    }
    return returnCode;
  }

  /**
   * Stop IHS http and admin server.
   * 
   * @return SUCCESS if all serers already stopped or stopped successfully.
   *         FAILURE otherwise
   */
  private int stopIHSServers()
  {
    boolean stopServerFailure = false;
    int returnCode = SUCCESS;
    IHSInstalledProduct ihsProduct;
    
    if (ivIsPLGFixpack)
    {
      ihsProduct = IHSInstalledProduct.getInstalledProduct(this, ivIHSLocation);
      if (ihsProduct == null)
      {
        setMessage("A version of IBM HTTP Server cannot be detected at " + ivIHSLocation + ".");
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
    }
    else
    {
      ihsProduct = (IHSInstalledProduct)ivMiddlewareProduct;
    }
    
    if (returnCode == SUCCESS)
    {
      if (ihsProduct.isServerRunning())
      {
        returnCode = ihsProduct.stopServer();
        if (returnCode == FAILURE)
        {
          setMessage(getResourceString(NLSKeys.HTTP_SERVER_STOP_FAILED));
          ivHelper.postErrorMessageToDeployerUI(this);
          stopServerFailure = true;
        }
      }
      if (ihsProduct.isAdminServerRunning())
      {
        returnCode = ihsProduct.stopAdminServer();
        if (returnCode == FAILURE)
        {
          setMessage(getResourceString(NLSKeys.HTTP_ADMIN_SERVER_STOP_FAILED));
          ivHelper.postErrorMessageToDeployerUI(this);
        }
      }
      if (stopServerFailure)
      {
        returnCode = FAILURE;
      }
    }
    return returnCode;
  }

  /**
   * Compare installed version of middleware with the version being installed.
   * 
   * @param String -
   *            installed version of the middleware
   * 
   * @return PDC_DOES_NOT_EXIST - if fix pack is installable at the specified
   *         location. PDC_EXISTS - if fix pack is already installed at the
   *         specified location. FAILURE - if fix pack is not installable at
   *         the specified location.
   */
  private int compareVersion(String installedVersion)
  {
    int returnCode = SUCCESS;
    // trim the version specified by user to first three digit. 
    //This version is conidered as a major version of the middleware for which fix pack installation is needed.
    String middlewareMajorVersion = ivMiddlewareProduct.trimVersionToNVersionPlaces(3, ivHelper
        .getProductInstallingVersion(this));
    // trim the installed version to first three digit
    String trimmedInstalledVersion = ivMiddlewareProduct.trimVersionToNVersionPlaces(3, installedVersion);
    // compare installed version with the version to be installed
    setVersion(installedVersion);
    setCompareVersion(ivHelper.getProductInstallingVersion(this));
    switch (ivHelper.compareVersions(this))
    {
      // version is lower than the version to be installed
      case -1:
      {
        // check if version is 6.1.0.x
        if (trimmedInstalledVersion.equalsIgnoreCase(middlewareMajorVersion))
        {
          //continue installation
          returnCode = PDC_DOES_NOT_EXIST;
        }
        else
        {
          //a lower version then 6.1.0.0 is installed
          returnCode = FAILURE;
        }
        break;
      }
        //versions are equal
      case 0:
      {
        returnCode = PDC_EXISTS;
        break;
      }
      case 1:
      {
        //later version already installed - no need to continue installation
        returnCode = FAILURE;
        break;
      }
    }
    return returnCode;
  }

  /**
   * Main. Checks the current state of the target machine for the requirements
   * and for any existing WAS Express installations.
   * 
   * @param args -
   *            expects no arguments
   */
  public static void main(String[] args)
  {
    //create and initialize the object
    WebSphereFamily6_1FixpackLnxPDC pdc = new WebSphereFamily6_1FixpackLnxPDC();

    //call init and terminate if it fails
    int returnCode = pdc.init(args);
    if (returnCode == SUCCESS)
    {
      returnCode = pdc.determineInstallAction();
    }
    System.exit(returnCode);
  }

}
