/**
 * WebSphereFamily6_1FixpackWinMainPDC.java
 * 
 * Licensed Materials - Property of IBM
 * 
 * 5724-J10
 * 
 * (C) Copyright IBM Corp. 2007
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * This is sample code made available for use in accordance with terms set forth
 * in the license agreement document for the IBM Express Runtime.
 */
package com.ibm.iru.webspherefamily6_1fixpackwin;

import java.io.File;
import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportWindowsBase;
import com.ibm.jsdt.support.SupportWindowsHelper;
import com.ibm.jsdt.support.installedproduct.IHSInstalledProduct;
import com.ibm.jsdt.support.installedproduct.WASExpressInstalledProduct;

public class WebSphereFamily6_1FixpackWinMain extends SupportWindowsBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private SupportWindowsHelper ivHelper;
  private WASExpressInstalledProduct ivWASProduct;
  private String ivMiddlewareLocation = null;
  private String ivUPDILocation = null;
  private String ivMiddlewareName = null;
  private String ivMaintanencePak = null;
  boolean ivIsWASFixpack = false;
  boolean ivIsIHSFixpack = false;
  boolean ivIsPLGFixpack = false;
  private String ivUnpackDir = null;
  private String ivUPDIResponseFile = null;
  //success string in the log file
  String SUCCESS_SEARCH_STRING = "INSTCONFSUCCESS";
  String WARNING_SEARCH_STRING = "INSTCONFPARTIALSUCCESS";
  //following are the prefix WebSphere family products fix pack name usually available with. Change this value
  //if fix pack file has specified with a different prefix
  String WAS_FIXPACK_FILE_PREFIX = "6.1.0-WS-WAS-WinX32";
  String IHS_FIXPACK_FILE_PREFIX = "6.1.0-WS-IHS-WinX32";
  String PLG_FIXPACK_FILE_PREFIX = "6.1.0-WS-PLG-WinX32";

  /**
   * Initializes the framework and validates arguments
   * Expect one arguments - Fix pack response file.
   * 
   * @return SUCCESS if initialization successful
   *         FAILURE if arguments incorrect or initialization failed
   */
  private int init(String[] args)
  {
    int no_Of_Expected_Args = 1;
    int returnCode = SUCCESS;
    ivHelper = getWindowsHelper();
    // set the message bundle for the resource strings
    setMainResources("com.ibm.iru.message.MessagesNLS");

    //ensure the right number of parameters was passed
    if (args.length != no_Of_Expected_Args)
    {
      String[] replacementText = { "WebSphereFamily6_1FixpackWinMain", String.valueOf(no_Of_Expected_Args),
          String.valueOf(args.length) };
      ivHelper.log(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      returnCode = FAILURE;
    }
    else
    {
      setResponseFileName(args[0].trim());
      //Express Runtime unpacked directory
      ivUnpackDir = ivHelper.getUnpackedDir(this);
      setPath(ivUnpackDir);
      ivUnpackDir = ivHelper.getWindowsShortPath(this).replace('\\', '/');
      //Update Installer resposne file location
      ivUPDIResponseFile = ivUnpackDir + ivHelper.getProductInstallingId(this) + SLASH
          + "responsefile.updiinstaller.txt";
      ivMiddlewareName = ivHelper.getIbmNsiPropValue("middlewareSelected");
      if (ivMiddlewareName.equalsIgnoreCase("WAS"))
      {
        ivIsWASFixpack = true;
        ivMiddlewareLocation = ivHelper.getIbmNsiPropValue("installationLocationWas");
      }
      else if (ivMiddlewareName.equalsIgnoreCase("IHS"))
      {
        ivIsIHSFixpack = true;
        ivMiddlewareLocation = ivHelper.getIbmNsiPropValue("installationLocationIhs");
      }
      else if (ivMiddlewareName.equalsIgnoreCase("PLG"))
      {
        ivIsPLGFixpack = true;
        ivMiddlewareLocation = ivHelper.getIbmNsiPropValue("installationLocationPlg");
      }
    }
    return returnCode;
  }

  /**
   * Look into the Express Runtime unpacked directory for the middleware fix pack name
   * 
   * @return SUCCESS if correct package name is provided.
   *         FAILURE if no or incorrect package name provided.
   */
  private int verifyPackageName()
  {
    int returnCode = SUCCESS;
    //get .pak files list from the ER unpacked directory
    String[] pakFilesArray = ivHelper.directoryGrep(ivUnpackDir, ".pak");
    if (pakFilesArray == null || pakFilesArray.length < 1)
    {
      setMessage("Could not find fix pack file with extension .pak.");
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    else if (pakFilesArray.length > 1)
    {
      boolean pakFound = false;
      for (int i = 0; i < pakFilesArray.length && !pakFound; i++)
      {
        if (ivIsWASFixpack)
        {
          if (pakFilesArray[i].indexOf(WAS_FIXPACK_FILE_PREFIX) > -1)
          {
            ivMaintanencePak = pakFilesArray[i];
            pakFound = true;
          }
        }
        else if (ivIsIHSFixpack)
        {
          if (pakFilesArray[i].indexOf(IHS_FIXPACK_FILE_PREFIX) > -1)
          {
            ivMaintanencePak = pakFilesArray[i];
            pakFound = true;
          }
        }
        else if (ivIsPLGFixpack)
        {
          if (pakFilesArray[i].indexOf(PLG_FIXPACK_FILE_PREFIX) > -1)
          {
            ivMaintanencePak = pakFilesArray[i];
            pakFound = true;
          }
        }
      }
      //fix pack file not found with the expected prefix. 
      if (!pakFound)
      {
        setMessage("Could not find fix pack file with expected name. Expected file name prefix is : "
            + WAS_FIXPACK_FILE_PREFIX + " for WebSphere Application Server - Express " + NEW_LINE
            + IHS_FIXPACK_FILE_PREFIX + " for IBM HTPP Server " + NEW_LINE + PLG_FIXPACK_FILE_PREFIX
            + " for WebSphere plugin.");
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
    }
    else
    {
      ivMaintanencePak = pakFilesArray[0];
    }
    return returnCode;
  }

  /**
   * Install Update Installer (UPDI). 
   * 1. Set UPDI install location in the response file. 
   * 2. Run Installer silently. 
   * 3. Verify that UPDI installed successfully.
   * 
   * @return SUCCESS if UPDI installation successfully completed
   *         FAILURE otherwise
   */
  private int installUPDI()
  {
    int returnCode = SUCCESS;
    boolean found = true;
    int count = 0;
    while (found)
    {
      ivUPDILocation = ivMiddlewareLocation + "UpdateInstallerER" + ++count;
      setFileName(ivUPDILocation);
      found = ivHelper.fileExists(this);
    }

    setFileName(ivUPDIResponseFile);
    setFind("-OPT installLocation=" + "\"" + "\"");
    setReplace("-OPT installLocation=" + "\"" + ivUPDILocation + "\"");
    ivHelper.fileSearchAndReplaceLine(this);

    //run installation
    String command = ivUnpackDir + "UpdateInstaller" + SLASH + "Install.exe -silent -options " + ivUPDIResponseFile;
    ivHelper.invokeCommandLogOutput(command);
    //verify success
    String updiInstallLog = ivUPDILocation + SLASH + "logs" + SLASH + "install" + SLASH + "log.txt";
    returnCode = verifyLogMessages(updiInstallLog);
    return returnCode;
  }

  /**
   * Install the fix pack. 
   * 1. Back up WebSphere profiles.
   * 2. Copy the appropriate .pak file in to the maintanence folder of the UPDI.
   * 3. Set installation location in the response file.
   * 4. Run install silently.
   * 5. Verify install.
   * 
   * @return SUCCESS if fix pack installation successfully completed
   *         FAILURE otherwise
   */
  private int installFixpack()
  {
    int returnCode = SUCCESS;
    if (ivIsWASFixpack)
    {
      ivWASProduct = WASExpressInstalledProduct.getInstalledProduct(this, ivMiddlewareLocation);
      if (ivWASProduct != null)
      {
        //Back up WebSphere profiles
        returnCode = ivWASProduct.backupWASProfilesConfiguration(ivHelper.getIbmNsiPropValue("appServerAdminUserName"),
            ivHelper.getIbmNsiPropValue("appServerAdminPassword"));
      }
    }
    if (returnCode == SUCCESS)
    {
      setFileName(getResponseFileName());
      //update the response file
      setFind("-W product.location=" + "\"" + "\"");
      setReplace("-W product.location=" + "\"" + ivMiddlewareLocation + "\"");
      ivHelper.fileSearchAndReplaceLine(this);
      setFind("-W maintenance.package=");
      setReplace("-W maintenance.package=" + "\"" + ivUPDILocation + SLASH + "maintenance" + SLASH + ivMaintanencePak
          + "\"");
      ivHelper.fileSearchAndReplaceLine(this);

      //copy the .pak file into the maintenance folder
      setSource(ivUnpackDir + SLASH + ivMaintanencePak);
      setTarget(ivUPDILocation + SLASH + "maintenance");
      ivHelper.fileCopy(this);

      String fixpakInstallLogDirName = ivMaintanencePak.substring(0, ivMaintanencePak.indexOf(".pak")) + ".install";
      String fixPackInstallLog = ivMiddlewareLocation + SLASH + "logs" + SLASH + "update" + SLASH
          + fixpakInstallLogDirName + SLASH + "updatelog.txt";
      setFileName(fixPackInstallLog);
      if (ivHelper.fileExists(this))
      {
        //RENAME the file. Log file is overwritten by the installer so in order to look for the correct success message
        //in the log file, rename the log file and delete it if exists before installation.
        setSource(fixPackInstallLog);
        setTarget(ivMiddlewareLocation + SLASH + "logs" + SLASH + "update" + SLASH + fixpakInstallLogDirName);
        setTargetFileName("updatelogER.bak");
        if (ivHelper.fileCopyAndRename(this))
        {
          ivHelper.fileDelete(this);
        }
      }

      /*
       * NOTE:
       * The update installer update.bat returns right after it invokes a java process to install the fix pack. So in 
       * order to wait for the java process to complete, we need to modify the update.bat file so that it return only
       * after the java process complete. In order to do so, we need to remove "start /B" from the update.bat file.
       */
      setFileName(ivUPDILocation + SLASH + "update.bat");
      setFind("start /B ");
      setReplace("");
      ivHelper.fileSearchAndReplace(this);

      setPath(ivUPDILocation);
      ivUPDILocation = ivHelper.getWindowsShortPath(this).replace('\\', '/');
      //run installation
      String command = ivUPDILocation + SLASH + "update.bat -options " + getResponseFileName() + " -silent";
      ivHelper.invokeCommandLogOutput(command);
      //verify success
      returnCode = verifyLogMessages(fixPackInstallLog);
    }
    else
    {
      setMessage("An error occurred when backing up WebSphere Application Server - Express profiles.  See log for more details.");
      ivHelper.log(this);
    }
    return returnCode;
  }

  /**
   * Presence of string INSTCONFSUCCESS in the installation log file indicates success.  
   * Presence of string INSTCONFPARTIALSUCCESS in the installation log file indicates partial success. 
   * 
   * @param  location - log location to search for success string.
   * @return SUCCESS if the installation is successful or partial successful.
   *         FAILURE if the installation is not successful.
   */
  private int verifyLogMessages(String logLocation)
  {
    int returnCode = FAILURE;
    //check to see if log file exists
    setFileName(logLocation);
    if (!ivHelper.fileExists(this))
    {
      setMessage(getResourceString("LOG_FILE_NOT_FOUND", new String[] { logLocation }));
      ivHelper.log(this);
    }
    else
    {
      // append to IBM Express Runtime log file
      ivHelper.logAppendFile(logLocation);
      File logFile = new File(logLocation);
      if (logFile != null)
      {
        //search for success message
        setGrepFileName(logLocation);
        setGrepString(SUCCESS_SEARCH_STRING);
        if (ivHelper.fileGrep(this))
        {
          returnCode = SUCCESS;
        }
        //search for any warning messages
        setGrepString(WARNING_SEARCH_STRING);
        if (ivHelper.fileGrep(this))
        {
          setMessage(getResourceString(NLSKeys.INSTALL_WARNING));
          ivHelper.postWarningMessageToDeployerUI(this);
          returnCode = SUCCESS;
        }
      }
    }
    return returnCode;
  }

  /**
   * Uninstall Update Installer (UPDI). 
   * 1. Run uninstaller silently. 
   * 2. Verify that UPDI uninstaled successfully.
   * 
   * @return SUCCESS if UPDI uninstallation completed successfully
   *         FAILURE otherwise
   */
  private int unInstallUPDI()
  {
    int returnCode = SUCCESS;
    //run installation
    String command = ivUPDILocation + SLASH + "uninstall" + SLASH + "uninstall.exe -silent";
    returnCode = ivHelper.invokeCommandLogOutput(command);
    //verify success
    String updiUninstallLog = ivUPDILocation + SLASH + "logs" + SLASH + "uninstall" + SLASH + "log.txt";
    returnCode = verifyLogMessages(updiUninstallLog);
    if (returnCode == SUCCESS)
    {
      //remove UPDI install directory
      File updiDirToDelete = new File(ivUPDILocation);
      if (!deleteDir(updiDirToDelete))
      {
        setMessage("An error occurred while cleaning the Update Installer installation directory.");
        ivHelper.postWarningMessageToDeployerUI(this);
        ivHelper.log(this);
      }
    }

    return returnCode;
  }

  /**
   * Deletes all files and subdirectories under dir.
   * Returns true if all deletions were successful.
   * If a deletion fails, the method stops attempting to delete and returns false.
   * 
   * @param File dir
   * 
   * @return true if all deletions were successful.
   *         if a deletion fails, the method stops attempting to delete and returns false.
   */
  private boolean deleteDir(File dir)
  {
    if (dir.isDirectory())
    {
      String[] subFiles = dir.list();
      if (subFiles.length > 0)
      {
        for (int i = 0; i < subFiles.length; i++)
        {
          boolean success = deleteDir(new File(dir, subFiles[i]));
          if (!success)
          {
            return false;
          }
        }
      }
    }
    // The directory is now empty so delete it
    return dir.delete();
  }

  /**
   * Installation verification steps:
   * If fix pack for, 
   * 1. WebSphere - run Installation Verification Test (IVT) provided by WebSphere.
   * 2. IBM HTTP Server - start HTPP server and admin serve if the user provided user id and password for admin server.
   * 
   * @return FAILURE if version of the middleware can not be determined
   *         SUCCESS otherwise with warning message if needed
   */

  private int verifyInstallation()
  {
    int returnCode = SUCCESS;
    if (ivIsWASFixpack)
    {
      //run IVT
      if (ivWASProduct.runInstallVerificationTestUsingDefualtProfileAndServer() == FAILURE)
      {
        setMessage(getResourceString(NLSKeys.WAS_VERIFICATION_FAIL));
        ivHelper.postWarningMessageToDeployerUI(this);
      }
    }
    if (ivIsIHSFixpack || ivIsPLGFixpack)
    {
      IHSInstalledProduct ihsProduct = null;
      if (ivIsIHSFixpack)
      {
        // start HTPP server and admin serve if the user provided user id and password for admin server
        ihsProduct = IHSInstalledProduct.getInstalledProduct(this, ivMiddlewareLocation);
      }
      if (ivIsPLGFixpack)
      {
        ihsProduct = IHSInstalledProduct.getInstalledProduct(this, ivHelper
            .getIbmNsiPropValue("installationLocationIhs"));
      }
      if (ihsProduct != null)
      {
        if (ihsProduct.startServer() != SUCCESS)
        {
          setMessage(getResourceString(NLSKeys.HTTP_SERVER_START_FAILED));
          ivHelper.postWarningMessageToDeployerUI(this);
        }
        if (ihsProduct.startAdminServer() != SUCCESS)
        {
          setMessage(getResourceString(NLSKeys.HTTP_ADMIN_SERVER_START_FAILED));
          ivHelper.postWarningMessageToDeployerUI(this);
        }

      }
    }
    return returnCode;
  }

  /**
   * Main. Perform required installations steps.
   * 
   * @param args - expects one arguments 
   *               1. Fix pack response file.
   */
  public static void main(String[] args)
  {
    //create and initialize the object
    WebSphereFamily6_1FixpackWinMain main = new WebSphereFamily6_1FixpackWinMain();

    //initialize class variables
    int returnCode = main.init(args);
    if (returnCode == SUCCESS)
    {
      //make sure that fix pack image is provided
      returnCode = main.verifyPackageName();
    }
    if (returnCode == SUCCESS)
    {
      //install update installer
      returnCode = main.installUPDI();
    }
    if (returnCode == SUCCESS)
    {
      //install fix pack
      returnCode = main.installFixpack();
    }
    if (returnCode == SUCCESS)
    {
      //uninstall update installer
      returnCode = main.unInstallUPDI();
    }
    if (returnCode == SUCCESS)
    {
      //verify using Installation Verification Test provided by WebSphere
      returnCode = main.verifyInstallation();
    }
    System.exit(returnCode);
  }
}
